"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
const express_1 = __importDefault(require("express"));
const config_1 = __importDefault(require("../config"));
const discovery_1 = require("@stargate-system/discovery");
const HUB_DISCOVERY_PORT = process.env.HUB_DISCOVERY_PORT ? Number.parseInt(process.env.HUB_DISCOVERY_PORT) : config_1.default.hubDiscoveryPort;
const DISCOVERY_KEYWORD = (_a = process.env.DISCOVERY_KEYWORD) !== null && _a !== void 0 ? _a : config_1.default.discoveryKeyword;
const DISCOVERY_INTERVAL = process.env.DISCOVERY_INTERVAL ? Number.parseInt(process.env.DISCOVERY_INTERVAL) : config_1.default.discoveryInterval;
const DISCOVERY_PORT = process.env.DISCOVERY_PORT ? Number.parseInt(process.env.DISCOVERY_PORT) : config_1.default.discoveryPort;
let isInitialized = false;
const initialize = () => {
    if (!isInitialized) {
        isInitialized = true;
        const app = (0, express_1.default)();
        const port = HUB_DISCOVERY_PORT;
        app.get('/', (req, res) => {
            var _a;
            const serverAddress = discovery_1.DefaultDiscoveryService.getServerAddress((_a = req.query.keyword) !== null && _a !== void 0 ? _a : DISCOVERY_KEYWORD);
            if (serverAddress) {
                res.send(serverAddress);
            }
            else {
                res.sendStatus(204);
            }
        });
        app.listen(port);
        discovery_1.DefaultDiscoveryService.setConfig({
            discoveryPort: DISCOVERY_PORT,
            discoveryInterval: DISCOVERY_INTERVAL
        });
        discovery_1.DefaultDiscoveryService.addServerAddressChangeListener((keyword) => {
            const serverAddress = discovery_1.DefaultDiscoveryService.getServerAddress(keyword);
            if (serverAddress) {
                console.log("Server " + keyword + " detected at " + serverAddress);
            }
            else {
                console.log("Server " + keyword + " offline");
            }
        });
    }
};
const getServerAddress = () => {
    return discovery_1.DefaultDiscoveryService.getServerAddress(DISCOVERY_KEYWORD);
};
const discoveryService = {
    initialize,
    getServerAddress
};
exports.default = discoveryService;
