"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LocalServerConnector = void 0;
const parser_readline_1 = require("@serialport/parser-readline");
const core_1 = require("@stargate-system/core");
const DiscoveryService_1 = __importDefault(require("./DiscoveryService"));
const DiscoveryService_2 = __importDefault(require("./DiscoveryService"));
const ws_1 = require("ws");
const config_1 = __importDefault(require("../config"));
class LocalServerConnector {
    constructor(serialPort) {
        this.handshakeDone = false;
        this.handshakeFailedAttempts = 0;
        this.portClosed = false;
        this.performHandshake = () => {
            this.timeout = setTimeout(() => {
                this.handshakeFailedAttempts++;
                if (this.handshakeFailedAttempts > 3) {
                    this.serialPort.close();
                }
                else {
                    this.performHandshake();
                }
            }, 1000);
            this.serialPort.write(core_1.MessageMapper.query(core_1.Keywords.type) + '\n');
        };
        this.handleSerialMessage = (message) => {
            if (!this.handshakeDone) {
                if (message === "*>type|6|device*+") {
                    if (this.timeout) {
                        clearTimeout(this.timeout);
                        this.timeout = undefined;
                    }
                    this.handshakeDone = true;
                    this.connectServer();
                }
            }
            else {
                if (this.forwardDeviceMessage) {
                    this.forwardDeviceMessage(message);
                }
            }
        };
        this.connectServer = () => {
            const DISCOVERY_INTERVAL = process.env.DISCOVERY_INTERVAL ? Number.parseInt(process.env.DISCOVERY_INTERVAL) : config_1.default.discoveryInterval;
            if (DiscoveryService_1.default.getServerAddress() === undefined) {
                this.timeout = setTimeout(this.connectServer, DISCOVERY_INTERVAL);
            }
            else {
                if (this.timeout) {
                    clearTimeout(this.timeout);
                    this.timeout = undefined;
                }
                const socket = new ws_1.WebSocket('ws://' + DiscoveryService_2.default.getServerAddress());
                const socketWrapper = {
                    send: () => { },
                    close: socket.close.bind(socket),
                    setOnClose: (callback) => socket.on('close', callback),
                    setOnMessage: (callback) => socket.on('message', (ev) => {
                        const message = ev.toString();
                        if (message.length > 0) {
                            this.serialPort.write(message + '\n');
                        }
                        callback(core_1.MessageMapper.acknowledge());
                    })
                };
                socket.onopen = () => {
                    this.forwardDeviceMessage = socket.send.bind(socket);
                    this.connection.setConnected(socketWrapper);
                };
                socket.on('error', console.log);
                socket.on('close', () => {
                    if (!this.portClosed) {
                        console.log('Reconnecting ' + this.serialPort.path);
                        this.forwardDeviceMessage = undefined;
                        setTimeout(this.connectServer, 5000);
                    }
                });
            }
        };
        this.serialPort = serialPort;
        this.connection = new core_1.DefaultConnection(true);
        this.connection.addStateChangeListener((state) => {
            if (state !== core_1.ConnectionState.ready) {
                this.serialPort.write(core_1.MessageMapper.command('notReady') + '\n');
            }
        });
        const parser = serialPort.pipe(new parser_readline_1.ReadlineParser({ delimiter: '\r\n' }));
        parser.on('data', (data) => {
            const message = data;
            this.handleSerialMessage(message);
        });
        serialPort.on('error', (err) => {
            console.log('Error: ', err.message);
        });
        serialPort.on('open', () => {
            console.log("Port opened " + serialPort.path);
            this.performHandshake();
        });
        serialPort.on('close', () => {
            this.portClosed = true;
            parser.destroy();
            this.connection.close();
            console.log("Port closed " + serialPort.path);
        });
    }
}
exports.LocalServerConnector = LocalServerConnector;
